# Generated by Django 2.2.11 on 2020-04-01 20:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0026_task_group'),
    ]

    operations = [
        migrations.CreateModel(
            name='RHSMCertGuard',
            fields=[
                ('contentguard_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='certguard_rhsmcertguard', serialize=False, to='core.ContentGuard')),
                ('ca_certificate', models.TextField()),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
            },
            bases=('core.contentguard',),
        ),
        migrations.CreateModel(
            name='X509CertGuard',
            fields=[
                ('contentguard_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='certguard_x509certguard', serialize=False, to='core.ContentGuard')),
                ('ca_certificate', models.TextField()),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
            },
            bases=('core.contentguard',),
        ),
    ]
